using Vintagestory.API.Client;
using Vintagestory.API.Common;
using Vintagestory.API.Datastructures;

namespace BodyHeatBarMod {
	public class BodyHeatBarSystem: ModSystem {
		protected BodyHeatElement element;

		public override bool ShouldLoad(EnumAppSide side) {
			return side == EnumAppSide.Client;
		}

		public override void StartClientSide(ICoreClientAPI capi) {
			this.element = new BodyHeatElement(capi);
		}
	}

	public class BodyHeatElement: HudElement {
		protected const int listenInterval = 200;
		protected const string dialogName = "bodyheat";

		protected const string barKey = "bodyheatbar";
		protected const float barX = -250;
		protected const float barY = -95;
		protected const float barWidth = 348;
		protected const float barHeight = 10;

		protected const float tempMin = 33;			// Deadly threshold.
		protected const float tempMax = 37;			// Default temperature.
		protected const float tempFlash = 34.5f;	// Shivering effect threshold.

		protected readonly double[] colour = { 0.5, 0.75, 0.8, 1 };

		protected BodyHeatStatbar bar;

		private long listenerId;
		private float lastTemp;

		public BodyHeatElement(ICoreClientAPI capi): base(capi) {
			this.listenerId = capi.Event.RegisterGameTickListener(this.OnGameTick, listenInterval);
			this.lastTemp = -1;

			// Create bar.
			ElementBounds dialogBounds = new ElementBounds() {
				Alignment = EnumDialogArea.CenterBottom,
				BothSizing = ElementSizing.Fixed,
				fixedWidth = barWidth,
				fixedHeight = barHeight
			};

			ElementBounds barBounds = ElementBounds.Fixed(barX, barY, barWidth, barHeight);

			Composers[dialogName] = capi.Gui
					.CreateCompo(dialogName, dialogBounds)
					.AddInteractiveElement(new BodyHeatStatbar(capi, barBounds, this.colour, true), barKey)
					.Compose();

			this.bar = (BodyHeatStatbar)Composers[dialogName].GetElement(barKey);
			this.bar.hideWhenFull = true;
			this.bar.SetMinMax(tempMin, tempMax);
			this.bar.SetLineInterval(0.5f);
			
			this.TryOpen();
		}

		private void OnGameTick(float dt) {
			// Update bar.
			if(this.bar == null) {
				return;
			}

			EntityPlayer entity = this.capi.World.Player.Entity;
			if(entity == null) {
				return;
			}

			ITreeAttribute tempTree = entity.WatchedAttributes.GetTreeAttribute("bodyTemp");
			if(tempTree == null) {
				return;
			}

			float temp = tempTree.GetFloat("bodytemp");

			this.bar.SetValue(temp);
			this.bar.shouldFlash = temp <= tempFlash;
			
			this.lastTemp = temp;
		}

		public override bool ShouldReceiveKeyboardEvents() {
			return false;
		}

		public override void OnMouseDown(MouseEvent args) {
			// Cannot be clicked.
		}

		public override void Dispose() {
			base.Dispose();

			capi.Event.UnregisterGameTickListener(this.listenerId);
		}

		protected override void OnFocusChanged(bool on) {
			// Cannot be focused.
		}
	}
}